/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>
#include <gax.h>
#include <gax_con.h>
#include <gax_tim.h>

#define	Oc	36		/* # of constellation matches */
#define	OO	16		/* # of discrete objects */

#define	OC_SET	~0	/* Constellation search not requested */
#define	OC_INC	 1	/* Constellation search requested - include */
#define	OC_EXC	 0	/* Constellation search requested - exclude */

static UB   z_a [16] = { 0 };
static UB inp_a [64];

SL	or_min_n =   0;				/* lower RA  */
SL	or_max_n =  24 * 3600 - 1;	/* upper RA  */
SL	od_min_n = -90 * 3600;		/* lower Dec */
SL	od_max_n =  90 * 3600;		/* upper Dec */

SL	b  ; /* box size (' of arc ) */
SL	Rb ; /* box size (" of time) */
SL	Db ; /* box size (" of arc ) */

UW	ox_min_n =     0;	/* lower Maj Dia */
UW	ox_max_n =     0;	/* upper Maj Dia */
UW	oy_min_n =     0;	/* lower Min Dia */
UW	oy_max_n =     0;	/* upper Min Dia */
UB	ov_min_n =   254;	/* lower Mag */
UB	ov_max_n =     0;	/* upper Mag */
UB	op_min_n =   254;	/* lower Mag */
UB	op_max_n =     0;	/* upper Mag */
UB	oa_min_n =   254;	/* lower Mag */
UB	oa_max_n =     0;	/* upper Mag */
UB	os_min_n =   254;	/* lower SB  */
UB	os_max_n =     0;	/* upper SB  */
UW	oz_min_n =     0;	/* lower Vel */
UW	oz_max_n = 65535;	/* upper Vel */
UB	ov_tmp_n;

UB	of_a [64] = { 0 };			/* output format string */
UB	o1_a [16] = {'M','N','I','U','G','E','K','A','V','C','P'};	/* match order */
UB	oo_a [64] = { 0 };			/* output .gxd filename */
UB	oc_a [Oc] = { 0 };			/* constellation filter */
UW	ou_b      = { 0 };			/* U2000.0 map # */

SB	oO_x [OO];					/* Object Catalog Index [CGX_xxx] */
UL	oO_a [OO];					/* Object Catalog Name */
UL	oO_n = 0;					/* Object Count */

UB	oC_N_b = 0, oC_I_b = 0, oC_U_b = 0, oC_G_b = 0, oC_E_b = 0, oC_P_b = 0;
UB	oC_C_b = 0, oC_A_b = 0, oC_K_b = 0, oC_V_b = 0, oC_M_b = 0;

UB	or_b = 0, ox_b = 0, oy_b = 0, ov_b = 0, oo_b = 0, oc_b = 0;
UB	oC_b = 0, oE_b = 0, oH_b = 0, om_b = 0, oz_b = 0, oN_b = 1;
UB	oc_x = OC_SET, oC_x = OC_SET;

UB	or_min_b = 0, or_max_b = 0, od_min_b = 0, od_max_b = 0;
UB	ox_min_b = 0, ox_max_b = 0, oy_min_b = 0, oy_max_b = 0;
UB	ov_min_b = 0, ov_max_b = 0, op_min_b = 0, op_max_b = 0;
UB	oa_min_b = 0, oa_max_b = 0, os_min_b = 0, os_max_b = 0;
UB	oz_min_b = 0, oz_max_b = 0;

SL	om_n = 0;

fp64_t	GMST0;
long_t	gmst0;
fp64_t	JD;

UL	year;
UL	date;
UB	month, dayom, dayoy;
SL	lat, lon, ele, hor;
UL	arc;
UL	lmt_hour, lmt_min, lmt_sec;
UL	gmt_hour, gmt_min, gmt_sec;

void_t outp (s)
ubyte_t	*s;
{
	while (*s) putchar (*s++);
	putchar (' ');
}

UB *name (gxd_t *d, UB o, UB x)
{
	 UB	 n = 0, *p = o1_a;

	 while (*p) {
	 	switch (*p) {
#ifdef GX_PGC
	 	  case 'P': if (d->pgc_a) n++; break;
#endif
#ifdef GX_NGC
	 	  case 'N': if (d->ngc_a) n++; break;
#endif
#ifdef GX_IC
	 	  case 'I': if (d-> ic_a) n++; break;
#endif
#ifdef GX_UGC
	 	  case 'U': if (d->ugc_a) n++; break;
#endif
#ifdef GX_MCG
	 	  case 'G': if (d->mcg_a) n++; break;
#endif
#ifdef GX_ESO
	 	  case 'E': if (d->eso_a) n++; break;
#endif
#ifdef GX_MK
	 	  case 'K': if (d-> mk_a) n++; break;
#endif
#ifdef GX_ARP
	 	  case 'A': if (d->arp_a) n++; break;
#endif
#ifdef GX_VV
	 	  case 'V': if (d-> vv_a) n++; break;
#endif
#ifdef GX_CGC
	 	  case 'C': if (d->cgc_a) n++; break;
#endif
#ifdef GX_M
	 	  case 'M': if (d->  m_a) n++; break;
#endif
		}
		if (n == x) break;
		p++;
	}
	switch (*p) {
#ifdef GX_PGC
 	  case 'P':  return ((UB *) b2a_nam (b2a_pgc (d->pgc_a, 1)));
#endif
#ifdef GX_NGC
 	  case 'N':  return ((UB *) b2a_nam (b2a_ngc (d->ngc_a, 1)));
#endif
#ifdef GX_IC
 	  case 'I':  return ((UB *) b2a_nam (b2a_ic  (d-> ic_a, 1)));
#endif
#ifdef GX_UGC
 	  case 'U':  return ((UB *) b2a_nam (b2a_ugc (d->ugc_a, 1)));
#endif
#ifdef GX_MCG
 	  case 'G':  return ((UB *) b2a_nam (b2a_mcg (d->mcg_a, 1)));
#endif
#ifdef GX_ESO
 	  case 'E':  return ((UB *) b2a_nam (b2a_eso (d->eso_a, 1)));
#endif
#ifdef GX_MK
 	  case 'K':  return ((UB *) b2a_nam (b2a_mk  (d-> mk_a, 1)));
#endif
#ifdef GX_ARP
 	  case 'A':  return ((UB *) b2a_nam (b2a_arp (d->arp_a, 1)));
#endif
#ifdef GX_VV
 	  case 'V':  return ((UB *) b2a_nam (b2a_vv  (d-> vv_a, 1)));
#endif
#ifdef GX_CGC
 	  case 'C':  return ((UB *) b2a_nam (b2a_cgc (d->cgc_a, 1)));
#endif
#ifdef GX_M
 	  case 'M':  return ((UB *) b2a_nam (b2a_m   (d->  m_a, 1)));
#endif
 	  default: return ((UB *) b2a_nam (" "));
	}
}

/*
 *	prnt - print data
 */
void_t prnt (gxd_t *d)
{
	 byte_t	*p = of_a;
	ubyte_t	 o, c;
	ulong_t	 i;

	if (*p == 0) return;

	while (c = *p++) {
	  o = 0;
	  switch (c) {
	    case '.': putchar (' '); break;
		case 'r': outp (i2a_ra  (d->raj_n)); break;
		case 'd': outp (i2a_dec (d->dcj_n)); break;
		case 'c': outp (con_i2a_nam (d->con_n)); break;
		case 'x': outp (i2a_maj (d->maj_n)); break;
		case 'y': outp (i2a_min (d->min_n)); break;
		case 'b': outp (i2a_pa  (d-> pa_n)); break;
#ifdef GX_Vel
		case 'z': outp (i2a_vel (d->vel_n)); break;
#endif
		case 's': outp (i2a_sb  (d-> sb_n)); break;
		case 'v': outp (i2a_mv  (d-> mv_n)); break;
		case 'p': outp (i2a_mp  (d-> mp_n)); break;
		case 'a': outp (i2a_ma  (d-> ma_n)); break;
		case 'u': outp (i2a_u2k (d->raj_n, d->dcj_n)); break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5': outp (name (d, o, c - '0')); break;
		
#ifdef GX_PGC
		case 'P': outp (b2a_pgc (d->pgc_a, o)); break;
#endif
#ifdef GX_NGC
		case 'N': outp (b2a_ngc (d->ngc_a, o)); break;
#endif
#ifdef GX_IC
		case 'I': outp (b2a_ic  (d-> ic_a, o)); break;
#endif
#ifdef GX_M
		case 'M': outp (b2a_m   (d->  m_a, o)); break;
#endif
#ifdef GX_CGC
		case 'C': outp (b2a_cgc (d->cgc_a, o)); break;
#endif
#ifdef GX_UGC
		case 'U': outp (b2a_ugc (d->ugc_a, o)); break;
#endif
#ifdef GX_MCG
		case 'G': outp (b2a_mcg (d->mcg_a, o)); break;
#endif
#ifdef GX_ESO
		case 'E': outp (b2a_eso (d->eso_a, o)); break;
#endif
#ifdef GX_MK
		case 'K': outp (b2a_mk  (d-> mk_a, o)); break;
#endif
#ifdef GX_ARP
		case 'A': outp (b2a_arp (d->arp_a, o)); break;
#endif
#ifdef GX_VV
		case 'V': outp (b2a_vv  (d-> vv_a, o)); break;
#endif

		case 'm': if (om_b) printf ("%4d ", (om_n + 30) / 60); break;

		case 'T': i = tim_transit (d->raj_n, lon, GMST0) + 0.5;
				  outp (i2a_hm (i));
				  break;

#if 0
		case 'R': i = tim_rise (d->raj_n, lon, GMST0, 34*60, lat, d->dcj_n) + 0.5;
				  outp (i2a_hm (i));
				  break;

		case 'S': i = tim_set  (d->raj_n, lon, GMST0, 34*60, lat, d->dcj_n) + 0.5;
				  outp (i2a_hm (i));
				  break;
#endif
	  }
	}
	putchar ('\n');
}

/*
 *	head - print header fields
 */
void_t head ()
{
	byte_t	*p = of_a;
	long_t	 o, c;

	while (c = *p++) {
	  o = 0;
	  switch (c) {
	    case '.': putchar (' '); break;
		case 'r': outp (hdr_ra  ( )); break;
		case 'd': outp (hdr_dec ( )); break;
		case 'c': outp (hdr_con ( )); break;
		case 'x': outp (hdr_maj ( )); break;
		case 'y': outp (hdr_min ( )); break;
#ifdef GX_Vel
		case 'z': outp (hdr_vel ( )); break;
#endif
		case 'b': outp (hdr_pa  ( )); break;
		case 's': outp (hdr_sb  ( )); break;
		case 'v': outp (hdr_mv  ( )); break;
		case 'p': outp (hdr_mp  ( )); break;
		case 'a': outp (hdr_ma  ( )); break;
		case 'u': outp (hdr_u2k ( )); break;
/*		case 'R': outp (hdr_ris ( )); break; */
/*		case 'S': outp (hdr_set ( )); break; */
		case 'T': outp (hdr_tra ( )); break;
/*		case 'e': outp (hdr_el  ( )); break; */
#ifdef GX_PGC
		case 'P': outp (hdr_pgc (o)); break;
#endif
#ifdef GX_NGC
		case 'N': outp (hdr_ngc (o)); break;
#endif
#ifdef GX_UGC
		case 'U': outp (hdr_ugc (o)); break;
#endif
#ifdef GX_IC
		case 'I': outp (hdr_ic  (o)); break;
#endif
#ifdef GX_MCG
		case 'G': outp (hdr_mcg (o)); break;
#endif
#ifdef GX_CGC
		case 'C': outp (hdr_cgc (o)); break;
#endif
#ifdef GX_ESO
		case 'E': outp (hdr_eso (o)); break;
#endif
#ifdef GX_MK
		case 'K': outp (hdr_mk  (o)); break;
#endif
#ifdef GX_ARP
		case 'A': outp (hdr_arp (o)); break;
#endif
#ifdef GX_VV
		case 'V': outp (hdr_vv  (o)); break;
#endif
#ifdef GX_M
		case 'M': outp (hdr_m   (o)); break;
#endif
		case '1':
		case '2':
		case '3':
		case '4':
		case '5': outp (hdr_nam (c)); break;
		case 'm': if (om_b) outp ("arc'"); break;
	  }
	}
	putchar ('\n');
}

/*
 *	dash - print header dashes
 */
void_t dash ()
{
	byte_t	*p = of_a;
	long_t	 o, c;

	while (c = *p++) {
	  o = 0;
	  switch (c) {
	    case '.': putchar (' '); break;
		case 'r': outp (dsh_ra  ( )); break;
		case 'd': outp (dsh_dec ( )); break;
		case 'c': outp (dsh_con ( )); break;
		case 'x': outp (dsh_maj ( )); break;
		case 'y': outp (dsh_min ( )); break;
#ifdef GX_Vel
		case 'z': outp (dsh_vel ( )); break;
#endif
		case 'b': outp (dsh_pa  ( )); break;
		case 's': outp (dsh_sb  ( )); break;
		case 'v': outp (dsh_mv  ( )); break;
		case 'p': outp (dsh_mp  ( )); break;
		case 'a': outp (dsh_ma  ( )); break;
/*		case 'R': outp (dsh_ris ( )); break; */
/*		case 'S': outp (dsh_set ( )); break; */
		case 'T': outp (dsh_tra ( )); break;
/*		case 'e': outp (dsh_el  ( )); break; */
		case 'u': outp (dsh_u2k ( )); break;
#ifdef GX_PGC
		case 'P': outp (dsh_pgc (o)); break;
#endif
#ifdef GX_NGC
		case 'N': outp (dsh_ngc (o)); break;
#endif
#ifdef GX_UGC
		case 'U': outp (dsh_ugc (o)); break;
#endif
#ifdef GX_IC
		case 'I': outp (dsh_ic  (o)); break;
#endif
#ifdef GX_MCG
		case 'G': outp (dsh_mcg (o)); break;
#endif
#ifdef GX_CGC
		case 'C': outp (dsh_cgc (o)); break;
#endif
#ifdef GX_ESO
		case 'E': outp (dsh_eso (o)); break;
#endif
#ifdef GX_MK
		case 'K': outp (dsh_mk  (o)); break;
#endif
#ifdef GX_ARP
		case 'A': outp (dsh_arp (o)); break;
#endif
#ifdef GX_VV
		case 'V': outp (dsh_vv  (o)); break;
#endif
#ifdef GX_M
		case 'M': outp (dsh_m   (o)); break;
#endif
		case '1':
		case '2':
		case '3':
		case '4':
		case '5': outp (dsh_nam ()); break;
		case 'm': if (om_b) outp ("----"); break;
	  }
	}
	putchar ('\n');
}

/*
 *	opts - parse options
 */
opts (UB *p)
{
	 long_t	 i, j;
	 byte_t	*s = p+2;

	if (*p == '@') {
		file (p+1);
		return;
	} else if (*p++ != '-') {
		 p--;
		 strcpy (inp_a, p);
		 return;
	}

	switch (*p++) {
	  case 'r':		/* Right Ascension */
		if (*p == 0) break;
	  	if (*p != ',') {      or_min_b=1; a2i_ra  (&p,&or_min_n); }
		if (*p == ',') { p++; or_max_b=1; a2i_ra  (&p,&or_max_n); }
		or_b = 1;
		break;
	  case 'd':		/* Declination */
		if (*p ==  0 ) break;
		if (*p != ',') {      od_min_b=1; a2i_dec (&p,&od_min_n); }
		if (*p == ',') { p++; od_max_b=1; a2i_dec (&p,&od_max_n); }
		or_b = 1;
		break;
	  case 'x':		/* Major Axis */
		if (*p ==  0 ) break;
		if (*p != ',') {      ox_min_b=1; a2i_maj (&p,&ox_min_n); }
		if (*p == ',') { p++; ox_max_b=1; a2i_maj (&p,&ox_max_n); }
		ox_b = 1;
		break;
	  case 'y':		/* Minor Axis */
		if (*p ==  0 ) break;
		if (*p != ',') {      oy_min_b=1; a2i_min (&p,&oy_min_n); }
		if (*p == ',') { p++; oy_max_b=1; a2i_min (&p,&oy_max_n); }
		oy_b = 1;
		break;
	  case 'v':		/* Visual Magnitude */
		if (*p ==  0 ) break;
		if (*p != ',') {      ov_min_b=1; a2i_mv  (&p,&ov_min_n); }
		if (*p == ',') { p++; ov_max_b=1; a2i_mv  (&p,&ov_max_n); }
		if (ov_min_b && ov_max_b) {
			ov_tmp_n = ov_min_n;
			ov_min_n = ov_max_n;
			ov_max_n = ov_tmp_n;
		}
		ov_b = 1;
		break;
	  case 'p':		/* Photographic Magnitude */
		if (*p ==  0 ) break;
		if (*p != ',') {      op_min_b=1; a2i_mp  (&p,&op_min_n); }
		if (*p == ',') { p++; op_max_b=1; a2i_mp  (&p,&op_max_n); }
		if (op_min_b && op_max_b) {
			ov_tmp_n = op_min_n;
			op_min_n = op_max_n;
			op_max_n = ov_tmp_n;
		}
		ov_b = 1;
		break;
	  case 'a':		/* Absolute Magnitude */
		if (*p ==  0 ) break;
		if (*p != ',') {      oa_min_b=1; a2i_ma  (&p,&oa_min_n); }
		if (*p == ',') { p++; oa_max_b=1; a2i_ma  (&p,&oa_max_n); }
		if (oa_min_b && oa_max_b) {
			ov_tmp_n = oa_min_n;
			oa_min_n = oa_max_n;
			oa_max_n = ov_tmp_n;
		}
		ov_b = 1;
		break;
	  case 's':		/* Surface Brightness */
		if (*p ==  0 ) break;
		if (*p != ',') {      os_min_b=1; a2i_sb  (&p,&os_min_n); }
		if (*p == ',') { p++; os_max_b=1; a2i_sb  (&p,&os_max_n); }
		ov_b = 1;
		break;
#ifdef GX_Vel
	  case 'z':		/* Velocity */
		if (*p ==  0 ) break;
		if (*p != ',') {      oz_min_b=1; a2i_vel (&p,&oz_min_n); }
		if (*p == ',') { p++; oz_max_b=1; a2i_vel (&p,&oz_max_n); }
		if (oz_min_b && oz_max_b) {
			ov_tmp_n = oz_min_n;
			oz_min_n = oz_max_n;
			oz_max_n = ov_tmp_n;
		}
		oz_b = 1;
		break;
#endif
	  case 'h':           help (p); break;
	  case 'f':           strcpy (of_a, p); break;
	  case '1':           strcpy (o1_a, p); break;
	  case 'o': oo_b = 1; strcpy (oo_a, p); break;
	  case 'N': oN_b = 0; break;

	  case 'L':		/* Observer Latitude */
		if (a2i_lat (&p, &lat)) {
			printf ("-L, invalid latitude\n");
			exit (0);
		}
		break;
	  case 'G':		/* Observer Longitude */
		if (a2i_lon (&p, &lon)) {
			printf ("-G, invalid longitude\n");
			exit (0);
		}
		break; 
	  case 'E':  ele = atoi (p);      oE_b = 1; break;
	  case 'H':  hor = atoi (p) * 60; oH_b = 1; break;
	  case 'm':  arc = atoi (p) * 60; om_b = 1; break;
	  case 'Y': year = atoi (p); break;
	  case 'D': month = *p++ - '0';
	  			month = (month * 10) + (*p++ - '0');
	  			dayom = *p++ - '0';
	  			dayom = (dayom * 10) + (*p++ - '0');
	  			break;
	  case 'u':		/* Uranometria 2000 Chart Number */
		if (ou_b) {
			printf ("-u already specified, 1 allowed\n");
			exit (0);
		}
		ou_b = atoi (p);
	  	if (u2k_box (ou_b,&or_min_n,&or_max_n,&od_min_n,&od_max_n)<0){
	  		printf ("-u%s, invalid map number\n", p);
	  		exit (0);
		}
		or_b = or_min_b = or_max_b = od_min_b = od_max_b = 1;
		break;
	  case 'O':		/* Single Object */
		if (oO_n >= OO) {
			printf ("-O%s, too many object names\n", p);
			exit (0);
		}
		oO_x [oO_n] = *p;
		if (!a2b_Cgx (&p, &oO_a [oO_n])) {
			printf ("-O%s, invalid object name\n", s);
			exit (0);
		}
		oO_n++;
		break;
	  case 'C':		/* Search Specific Catalog */
	    if (*p == '!') {
	    	if (oC_x == OC_INC) {
	    		printf ("-C%s can't be mixed with -C\n", p);
	    		exit (0);
			}
			oC_x = OC_EXC;
			p++;
	    } else {
	    	if (oC_x == OC_EXC) {
	    		printf ("-C%s can't be mixed with -C!\n", p);
	    		exit (0);
			}
			oC_x = OC_INC;
	    }
		while (*p) {
			switch (*p++) {
#ifdef GX_PGC
			  case 'P': oC_P_b = 1; oC_b = 1; break;
#endif
#ifdef GX_NGC
			  case 'N': oC_N_b = 1; oC_b = 1; break;
#endif
#ifdef GX_IC
			  case 'I': oC_I_b = 1; oC_b = 1; break;
#endif
#ifdef GX_UGC
			  case 'U': oC_U_b = 1; oC_b = 1; break;
#endif
#ifdef GX_MCG
			  case 'G': oC_G_b = 1; oC_b = 1; break;
#endif
#ifdef GX_ESO
			  case 'E': oC_E_b = 1; oC_b = 1; break;
#endif
#ifdef GX_CGC
			  case 'C': oC_C_b = 1; oC_b = 1; break;
#endif
#ifdef GX_ARP
			  case 'A': oC_A_b = 1; oC_b = 1; break;
#endif
#ifdef GX_MK
			  case 'K': oC_K_b = 1; oC_b = 1; break;
#endif
#ifdef GX_VV
			  case 'V': oC_V_b = 1; oC_b = 1; break;
#endif
#ifdef GX_M
			  case 'M': oC_M_b = 1; oC_b = 1; break;
#endif
			  default: printf ("-C%c unknown\n", *(p-1)); exit (0);
			}
		}
		break;
	  case 'c':		/* Constellation */
	    if (*p == '!') {
	    	if (oc_x == OC_INC) {
	    		printf ("-c%s can't be mixed with -c\n", p);
	    		exit (0);
			}
			oc_x = OC_EXC;
			p++;
	    } else {
	    	if (oc_x == OC_EXC) {
	    		printf ("-c%s can't be mixed with -c!\n", p);
	    		exit (0);
			}
			oc_x = OC_INC;
	    }
	  	if (oc_b >= Oc) {
	  		if (oc_x == OC_EXC) p--;
	  		printf ("-c%s too many constellations\n", p);
	  		exit (0);
		}
		if (!(j = con_a2i_nam (p))) {
	  		if (oc_x == OC_EXC) p--;
			printf ("-c%s unknown constellation\n", p);
			exit (0);
		}
		oc_a [oc_b++] = j;
		break;

	  case 'V':		/* Version Info */
	    gax_ver ("gax");
	    exit (0);

	  default: printf ("-%c unknown\n", *(p-1)); exit (0);
	}
}

/*
 *	check - check sanity of option combinations
 */
check ()
{
	UL	 i;
	time_t	time_n;
	struct	tm	*tm_p;

	if (om_b) {
		i = 0;
		if (!or_min_b) { printf ("-r min must be specified with -m\n"); i=1; }
		if (!od_min_b) { printf ("-d min must be specified with -m\n"); i=1; }
		if ( or_max_b) { printf ("-r max cant be specified with -m\n"); i=1; }
		if ( od_max_b) { printf ("-d max cant be specified with -m\n"); i=1; }
		if (i) exit (0);
		or_b = or_min_b = or_max_b = od_min_b = od_max_b = 0;
	}
	if (or_b && or_min_n > or_max_n)
		or_b = 2;

	  time_n = time (0);
	    tm_p = localtime (&time_n);
  	lmt_hour = tm_p->tm_hour;
  	lmt_min  = tm_p->tm_min;
  	lmt_sec  = tm_p->tm_sec;
	if ( year == 0) year  = tm_p->tm_year + 1900;
	if (month == 0) {
		month = tm_p->tm_mon + 1;
		dayom = tm_p->tm_mday;
		dayoy = tm_p->tm_yday;
	}
	JD = tim_DMY2JD (dayom, month, year);
	GMST0 = tim_JD2GMST0 (JD);
	gmst0 = GMST0 + 0.5;

	    tm_p = gmtime (&time_n);
  	gmt_hour = tm_p->tm_hour;
  	gmt_min  = tm_p->tm_min;
  	gmt_sec  = tm_p->tm_sec;
}

/*
 *	file - read commands from a file
 */
file (UB *p)
{
	UL	 fd;
	UB	 str [128], *c = str;

	if ((fd = open (p, O_RDONLY, 0666)) == -1) {
		printf ("unable to open @ file %s\n", p);
		exit (0);
	}
	while (read (fd, c, 1) == 1) {
		switch (*c) {
		  case '\n':
		  case '\t':
		  case ' ':
			if (c != str) {
				*c = 0;
				opts (c = str);
			}
			break;
		  default:
			c++;
			break;
		}
	}
	if (c > str) { *c = 0; opts (str); }
	close (fd);
}

/*
 *	fltr - filter object based on search criteria
 */
fltr (gxd_t *d)
{
	UL	 n = 0, i, j;
	UB	*p;

	if (om_b) {
		if (abs (d->dcj_n - od_min_n) > arc) return (0);
		if ((om_n = ang_sep (d->raj_n, d->dcj_n, or_min_n, od_min_n)) > arc)
			return (0);
	}
POS:
	if (or_b) {
		if (or_b == 1) {
			if (d->raj_n < or_min_n) return (0);
			if (d->raj_n > or_max_n) return (0);
		} else {
			if (d->raj_n < or_min_n &&
				d->raj_n > or_max_n) return (0);
		}
		if (d->dcj_n < od_min_n) return (0);
		if (d->dcj_n > od_max_n) return (0);
	}
SIZ:
	if (ox_b) {
		if (ox_min_b && d->maj_n < ox_min_n) return (0);
		if (ox_max_b && d->maj_n > ox_max_n) return (0);
	}
	if (oy_b) {
		if (oy_min_b && d->min_n < oy_min_n) return (0);
		if (oy_max_b && d->min_n > oy_max_n) return (0);
	}
#ifdef GX_Vel
VEL:
	if (oz_b) {
		if (oz_min_b && d->vel_n < oz_min_n) return (0);
		if (oz_max_b && d->vel_n > oz_max_n) return (0);
	}
#endif
MAG:
	if (ov_b) {
		if ((ov_min_b || ov_max_b) && !d->mv_n) return (0);
		if ((op_min_b || op_max_b) && !d->mp_n) return (0);
		if ((oa_min_b || oa_max_b) && !d->ma_n) return (0);
		if ((os_min_b || os_max_b) && !d->sb_n) return (0);
	
		if (ov_min_b && d->mv_n > ov_min_n) return (0);
		if (ov_max_b && d->mv_n < ov_max_n) return (0);
		if (op_min_b && d->mp_n > op_min_n) return (0);
		if (op_max_b && d->mp_n < op_max_n) return (0);
		if (oa_min_b && d->ma_n > oa_min_n) return (0);
		if (oa_max_b && d->ma_n < oa_max_n) return (0);
		if (os_min_b && d->sb_n > os_min_n) return (0);
		if (os_max_b && d->sb_n < os_max_n) return (0);
	}

CAT:
	if (oC_b) {
#ifdef GX_PGC
		if (oC_P_b) { i = d->pgc_a; goto CAT1; }
#endif
#ifdef GX_NGC
		if (oC_N_b) { i = d->ngc_a; goto CAT1; }
#endif
#ifdef GX_IC
		if (oC_I_b) { i = d-> ic_a; goto CAT1; }
#endif
#ifdef GX_UGC
		if (oC_U_b) { i = d->ugc_a; goto CAT1; }
#endif
#ifdef GX_MCG
		if (oC_G_b) { i = d->mcg_a; goto CAT1; }
#endif
#ifdef GX_CGC
		if (oC_C_b) { i = d->cgc_a; goto CAT1; }
#endif
#ifdef GX_ESO
		if (oC_E_b) { i = d->eso_a; goto CAT1; }
#endif
#ifdef GX_MK
		if (oC_K_b) { i = d-> mk_a; goto CAT1; }
#endif
#ifdef GX_ARP
		if (oC_A_b) { i = d->arp_a; goto CAT1; }
#endif
#ifdef GX_VV
		if (oC_V_b) { i = d-> vv_a; goto CAT1; }
#endif
#ifdef GX_M
		if (oC_M_b) { i = d->  m_a; goto CAT1; }
#endif

CAT1:
		if (oC_x == OC_INC && !i) return (0);
		if (oC_x == OC_EXC &&  i) return (0);
	}
OBJ:
	if (oO_n) {
		for (n = 0; n < oO_n; n++) {
			switch (oO_x [n]) {
#ifdef GX_PGC
			  case 'P': i=d->pgc_a; break;
#endif
#ifdef GX_NGC
			  case 'N': i=d->ngc_a; break;
#endif
#ifdef GX_IC
			  case 'I': i=d-> ic_a; break;
#endif
#ifdef GX_UGC
			  case 'U': i=d->ugc_a; break;
#endif
#ifdef GX_MCG
			  case 'G': i=d->mcg_a; break;
#endif
#ifdef GX_ESO
			  case 'E': i=d->eso_a; break;
#endif
#ifdef GX_CGC
			  case 'C': i=d->cgc_a; break;
#endif
#ifdef GX_ARP
			  case 'A': i=d->arp_a; break;
#endif
#ifdef GX_MK
			  case 'K': i=d-> mk_a; break;
#endif
#ifdef GX_VV
			  case 'V': i=d-> vv_a; break;
#endif
#ifdef GX_M
			  case 'M': i=d->  m_a; break;
#endif
			}
			if (i == oO_a [n]) goto CON;
		}
		return (0);
	}
CON:
	if (oc_b) {
		for (n = 0; n < oc_b; n++)
			if (oc_a [n] == d->con_n)
				return (oc_x);

		if (n == oc_b) return (!oc_x);
	}

	return (1);
}

UB *month_a [] = {
	"   ", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
	       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

/*
 *	popt - print selected options
 */
popt ()
{
	UL	 i;

	if (om_b) {
		printf ("     RA: [ %s]\n",	i2a_ra  (or_min_n));
		printf ("    Dec: [%s]\n",	i2a_dec (od_min_n));
		printf ("    Arc: [%d']\n",  arc / 60);
	}
	if (or_min_b || or_max_b) {
		printf ("     RA: [ %s",	i2a_ra  (or_min_n));
		printf (", %s]\n",			i2a_ra  (or_max_n));
	}
	if (od_min_b || od_max_b) {
		printf ("    Dec: [%s",		i2a_dec (od_min_n));
		printf (",%s]\n",			i2a_dec (od_max_n));
	}
	if (ox_min_b || ox_max_b) {
		printf ("    Maj: [  %s",	i2a_maj (ox_min_n));
		printf (",  %s]\n",			i2a_maj (ox_max_n));
	}
	if (oy_min_b || oy_max_b) {
		printf ("    Min: [  %s",	i2a_min (oy_min_n));
		printf (",  %s]\n",			i2a_min (oy_max_n));
	}
	if (ov_min_b || ov_max_b) {
		printf ("     Mv: [   %s",	i2a_mv  (ov_max_n));
		printf (",   %s]\n",		i2a_mv  (ov_min_n));
	}
	if (ou_b) printf ("    U2k: [%d]\n", ou_b);
	if (oc_b) {
		printf ("  ");
		putchar ((oc_x) ? ' ' : '!');
		printf (" Con:");
		for (i = 0; i < oc_b; i++) {
			if (i && !(i % 12)) printf ("\n        ");
			printf (" %s", con_i2a_nam (oc_a [i]));
		}
		printf ("\n");
	}
	if (oC_b) {
		printf ("    Cat:");
		if (oC_P_b) printf (" PGC");
		if (oC_M_b) printf (" M");
		if (oC_N_b) printf (" NGC");
		if (oC_I_b) printf (" IC");
		if (oC_U_b) printf (" UGC");
		if (oC_G_b) printf (" MCG");
		if (oC_E_b) printf (" ESO");
		if (oC_K_b) printf (" MK");
		if (oC_A_b) printf (" ARP");
		if (oC_V_b) printf (" VV");
		printf ("\n");
	}
	printf ("   DATE: [%02d%s%d]\n", dayom, month_a [month], year);
	printf ("     JD: [%.2lf]\n", JD);
	printf ("    LMT: [%02d%02d%02d]\n", lmt_hour, lmt_min, lmt_sec);
	printf ("    GMT: [%02d%02d%02d]\n", gmt_hour, gmt_min, gmt_sec);
	printf ("  GMST0: [%s]\n", i2a_ra (gmst0));
	printf ("\n");
}

main (long_t ac, byte_t *av [])
{
	 long_t	 i, fdi_n, fdo_n;
	  gxd_t	 d [1];

	if (ac < 2) help (0);

	for (i = 1; i < ac; i++)
		opts (av [i]);

	check ();

	if (!(fdi_n = gxd_open (inp_a, O_RDONLY, 0666))) {
		printf ("unable to open input file %s\n", inp_a);
		exit (-1);
	}
	if (oo_b) {
		if (!(fdo_n = gxd_open (oo_a, O_CREAT | O_TRUNC | O_WRONLY, 0666))) {
			printf ("unable to open output file %s\n", oo_a);
			exit (-1);
		}

	}

	if (oN_b) {		/* print a header ? */
		popt ();
		head ();
		dash ();
	}

	while (gxd_read (fdi_n, d, 1)) {
		if (fltr (d)) {
			if (oo_b) gxd_write (fdo_n, d, 1);
	    	prnt (d);
		}
	}

	          gxd_close (fdi_n);
	if (oo_b) gxd_close (fdo_n);
}

