/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <fcntl.h>
#include <stdio.h>
#include <gax_cat.h>

/*
 *	a2b_n - Convert ASCII to BCD - (n) One Digit
 */
static a2b_n (UB **p, UL *b)
{
	if (**p == ' ') { (*p)++; return (0); }

	if (!isdigit (**p)) return (1);

	*b = (*b << 4) + (**p - '0');

	(*p)++;

	return (0);
}

/*
 *	a2b_nn - Convert ASCII to BCD - (nn) Two Digits
 */
static a2b_nn  (UB **p, UL *b)
{
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	return (0);
}

/*
 *	a2b_nnn - Convert ASCII to BCD - (nnn) Three Digits
 */
static a2b_nnn (UB **p, UL *b)
{
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	return (0);
}

/*
 *	a2b_a - Convert ASCII to BCD - (a) One Letter [a-f] or [A-F]
 */
static long_t a2b_a (UB **p, UL *b)
{
	UB	c = **p;

	if (c ==  0 )             { (*p)++; *b = (*b<<4);               return (0); }
	if (c == ' ')             { (*p)++; *b = (*b<<4);               return (0); }
	if (c >= 'A' && c <= 'F') { (*p)++; *b = (*b<<4) + (c - 'A'+1); return (0); }
	if (c >= 'a' && c <= 'f') { (*p)++; *b = (*b<<4) + (c - 'a'+1); return (0); }
	return (1);
}

/*
 *========================================================================
 *	a2b_??????a - Object Name Catalog Input
 *========================================================================
 */

/*
 *	a2b_nnna - Convert Catalog Name
 */
static a2b_nnna (UB **p, UL *b)
{
	*b = 0;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_a (p, b)) return (1);
	return (0);
}

/*
 *	a2b_nnnna - Convert Catalog Name
 */
static a2b_nnnna (UB **p, UL *b)
{
	*b = 0;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_a (p, b)) return (1);
	return (0);
}

/*
 *	a2b_nnnnn - Convert Catalog Name
 */
static a2b_nnnnn (UB **p, UL *b)
{
	*b = 0;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	return (0);
}

/*
 *	a2b_nnnnna - Convert Catalog Name
 */
static a2b_nnnnna (UB **p, UL *b)
{
	if (a2b_nnnnn (p, b)) return (1);
	if (a2b_a     (p, b)) return (1);
	return (0);
}

/*
 *	a2b_nnn_nnna - Convert Catalog Name
 */
static a2b_nnn_nnna (UB **p, UL *b)
{
	*b = 0;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (**p!=' '  &&  **p!='-') return (1); (*p)++;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_a (p, b)) return (1);
	return (0);
}

/*
 *	a2b_nn_nn_nnna - Convert Catalog Name
 */
static a2b_nn_nn_nnna (UB **p, SL *b)
{
	UL	x;

	*b = 0;

	if (**p == ' ') return (0);

	x = (**p == '-') ? 0x80000000 : 0x00;   (*p)++;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (**p!=' '  &&  **p!='-') return (1); (*p)++;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (**p!=' '  &&  **p!='-') return (1); (*p)++;
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_n (p, b)) return (1);
	if (a2b_a (p, b)) return (1);
	*b |= x;
	return (0);
}

#define	II(x,y,z) return (x (y, z))
a2b_pgc (UB **p, UL *b) { II (a2b_nnnnn,     p, b); }
a2b_ngc (UB **p, UL *b) { II (a2b_nnnna,      p, b); }
a2b_ic  (UB **p, UL *b) { II (a2b_nnnna,      p, b); }
a2b_ugc (UB **p, UL *b) { II (a2b_nnnnna,     p, b); }
a2b_mcg (UB **p, UL *b) { II (a2b_nn_nn_nnna, p, b); }
a2b_eso (UB **p, UL *b) { II (a2b_nnn_nnna,   p, b); }
a2b_cgc (UB **p, UL *b) { II (a2b_nnn_nnna,   p, b); }
a2b_arp (UB **p, UL *b) { II (a2b_nnnna,      p, b); }
a2b_mk  (UB **p, UL *b) { II (a2b_nnnna,      p, b); }
a2b_vv  (UB **p, UL *b) { II (a2b_nnnna,      p, b); }
a2b_m   (UB **p, UL *b) { II (a2b_nnn,        p, b); }
#undef	II
 
a2b_Cgx (UB **p, UL *b)
{
	UB	 c = **p;

	(*p)++;
	switch (c) {
#ifdef GX_NGC
	  case 'N': return (a2b_ngc (p, b) ? 0 : CGX_NGC); break;
#endif
#ifdef GX_IC
	  case 'I': return (a2b_ic  (p, b) ? 0 : CGX_IC ); break;
#endif
#ifdef GX_UGC
	  case 'U': return (a2b_ugc (p, b) ? 0 : CGX_UGC); break;
#endif
#ifdef GX_ESO
	  case 'E': return (a2b_eso (p, b) ? 0 : CGX_ESO); break;
#endif
#ifdef GX_CGC
	  case 'C': return (a2b_cgc (p, b) ? 0 : CGX_CGC); break;
#endif
#ifdef GX_VV
	  case 'V': return (a2b_vv  (p, b) ? 0 : CGX_VV ); break;
#endif
#ifdef GX_ARP
	  case 'A': return (a2b_arp (p, b) ? 0 : CGX_ARP); break;
#endif
#ifdef GX_PGC
	  case 'P': return (a2b_pgc (p, b) ? 0 : CGX_PGC); break;
#endif
#ifdef GX_MCG
	  case 'G': return (a2b_mcg (p, b) ? 0 : CGX_MCG); break;
#endif
#ifdef GX_MK
	  case 'K': return (a2b_mk  (p, b) ? 0 : CGX_MK ); break;
#endif
#ifdef GX_M
	  case 'M': (*p)++;
	  case '1': case '2': case '3': case '4': case '5':
	  case '6': case '7': case '8': case '9': case '0':
	  			(*p)--;
	  			return (a2b_m   (p, b) ? 0 : CGX_M  ); break;
#endif
	}
	return (0);
}

#ifdef TEST

#define	X(B,x)	((B >> x) & 0x0F)
test (UB *s, UL b)
{
	printf ("%20s: ", s);

	putchar (X (b, 28) + '0');
	putchar (X (b, 24) + '0');
	putchar (X (b, 20) + '0');
	putchar (X (b, 16) + '0');
	putchar (X (b, 12) + '0');
	putchar (X (b,  8) + '0');
	putchar (X (b,  4) + '0');
	putchar (X (b,  0) + '0');
	putchar ('\n');
}

main ()
{
	SL	 b;
	UB	*p;

	b=0; p="5";           a2b_n          (&p, &b); test ("a2b_5",           b);
	b=0; p="57";          a2b_nn         (&p, &b); test ("a2b_57",          b);
	b=0; p="357";         a2b_nnn        (&p, &b); test ("a2b_357",         b);
	b=0; p="c";           a2b_a          (&p, &b); test ("a2b_c",           b);
	b=0; p="123a";        a2b_nnna       (&p, &b); test ("a2b_123a",        b);
	b=0; p="1234b";       a2b_nnnna      (&p, &b); test ("a2b_1234b",       b);
	b=0; p="12345c";      a2b_nnnnna     (&p, &b); test ("a2b_12345c",      b);
	b=0; p="123-456d";    a2b_nnn_nnna   (&p, &b); test ("a2b_123-456d",    b);
	b=0; p="+12-34-567e"; a2b_nn_nn_nnna (&p, &b); test ("a2b_+12-34-567e", b);
	b=0; p="-12-34-567e"; a2b_nn_nn_nnna (&p, &b); test ("a2b_-12-34-567e", b);
}
#endif
