/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <fcntl.h>
#include <stdio.h>
#include <gax_cat.h>

void_t a2i_a2e (UB *b, UB *p)
{
	fprintf (stderr, "%s\n", b);
	while (p-- != b) fputc (' ',  stderr);
					 fputc ('^',  stderr);
					 fputc ('\n', stderr);
}

/*
 *	a2i_i - Convert ASCII to Integer
 */
static a2i_i (UB **p, UL *b)
{
	if (**p == ' ') { (*p)++; return (0); }

	if (!isdigit (**p)) return (1);

	*b = (*b * 10) + (**p - '0');

	(*p)++;

	return (0);
}

/*
 *	a2i_ii - Convert ASCII to Integer
 */
static a2i_ii (UB **p, UL *b)
{
	if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	return (0);
}

/*
 *	a2i_iii - Convert ASCII to fixed point (nnn)
 */
static a2i_iii (UB **p, UL *b)
{
	if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	return (0);
}

/*
 *	a2i_iiiii - Convert ASCII to fixed point (nnnnn)
 */
static a2i_iiiii (UB **p, UL *b)
{
	if (**p == '-') goto zero; if (a2i_i (p, b)) return (1);
	if (**p == '-') goto zero; if (a2i_i (p, b)) return (1);
	if (**p == '-') goto zero; if (a2i_i (p, b)) return (1);
	if (**p == '-') goto zero; if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	return (0);
zero:
	*b = 0;
	return (0);
}

/*
 *	a2i_ii_i - Convert ASCII to fixed point (nn.n)
 */
static a2i_ii_i (UB **p, UL *b)
{
	if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	if (**p!='.'  &&  **p!=' ') return (1); (*p)++;
	if (a2i_i  (p, b)) return (1);
	return (0);
}

/*
 *	a2i_iii_i - Convert ASCII to fixed point (nnn.n)
 */
static a2i_iii_i (UB **p, UL *b)
{
	if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	if (a2i_i (p, b)) return (1);
	if (**p!='.'  &&  **p!=' ') return (1); (*p)++;
	if (a2i_i   (p, b)) return (1);
	return (0);
}

#define	II(x,y,z) UL i; i = 0; if (x (y, &i)) return (1); *z = i; return (0)
a2i_maj (UB **p, UW *b) { II (a2i_iii_i, p, b); }
a2i_min (UB **p, UW *b) { II (a2i_iii_i, p, b); }
a2i_vel (UB **p, UW *b) { II (a2i_iiiii, p, b); }
a2i_mv  (UB **p, UB *b) { II (a2i_ii_i,  p, b); }
a2i_mp  (UB **p, UB *b) { II (a2i_ii_i,  p, b); }
a2i_ma  (UB **p, UB *b) { II (a2i_ii_i,  p, b); }
a2i_sb  (UB **p, UB *b) { II (a2i_ii_i,  p, b); }
#undef	II

a2i_ra (UB **p, SL *rp)
{
	UL	h = 0, m = 0, s = 0;

	if (a2i_ii (p, &h)) return (1);
	if (a2i_ii (p, &m)) return (1);
	if (a2i_ii (p, &s)) return (1);

	*rp = (((h * 60) + m) * 60) + s;

	return (0);
}

a2i_dec (UB **p, SL *dp)
{
	UL	d = 0, m = 0, s = 0;
	SB	x;

	if      (**p == '+') x =  1;
	else if (**p == ' ') x =  1;
	else if (**p == '-') x = -1;
	else return (-1);

	(*p)++;
	
	if (a2i_ii (p, &d)) return (1);
	if (a2i_ii (p, &m)) return (1);
	if (a2i_ii (p, &s)) return (1);

	*dp = x * ((((d * 60) + m) * 60) + s);

	return (0);
}

a2i_lat (UB **p, SL *dp)
{
	UL	d = 0, m = 0, s = 0;
	SB	x;

	if      (**p == '+') { x =  1; (*p)++; }
	else if (**p == '-') { x = -1; (*p)++; }
	else x = 1;

	if (a2i_ii (p, &d)) return (1);
	if (a2i_ii (p, &m)) return (1);
	if (a2i_ii (p, &s)) return (1);

	*dp = x * ((((d * 60) + m) * 60) + s);

	return (0);
}

a2i_lon (UB **p, SL *dp)
{
	UL	d = 0;
	UL	m = 0, s = 0;
	SB	x;

	if      (**p == '+') { x =  1; (*p)++; }
	else if (**p == '-') { x = -1; (*p)++; }
	else x = 1;

	if (a2i_iii (p, &d)) return (1);
	if (a2i_ii  (p, &m)) return (1);
	if (a2i_ii  (p, &s)) return (1);

	*dp = x * ((((d * 60) + m) * 60) + s);

	return (0);
}

a2i_pa  (UB **p, UB *b)
{
	UL	i = 0;

	if (!strncmp (*p, "   ", 3)) {		/* blank == ~0 */
		*b = ~0;
		return (0);
	}
	if (a2i_iii (p, &i)) return (1);
	*b = i;
	return (0);
}

#ifdef TEST
main ()
{
	UB	*p;
	UW	 w;
	UB	 b;
	UL	 l;

	l=0; p =       "7"; a2i_i   (&p,&l); printf ("  a2i_i: (      7) %d\n", l);
	l=0; p =      "57"; a2i_ii  (&p,&l); printf (" a2i_ii: (     57) %d\n", l);
	l=0; p =     "357"; a2i_iii (&p,&l); printf ("a2i_iii: (    357) %d\n", l);
	w=0; p =   "123.4"; a2i_maj (&p,&w); printf ("    maj: (  123.4) %d\n", w);
	w=0; p =   "  1.2"; a2i_min (&p,&w); printf ("    min: (    1.2) %d\n", w);
	b=0; p =    "12.5"; a2i_mv  (&p,&b); printf ("     mv: (   12.5) %d\n", b);
	b=0; p =    "12.6"; a2i_mp  (&p,&b); printf ("     mp: (   12.6) %d\n", b);
	b=0; p =    "12.7"; a2i_ma  (&p,&b); printf ("     ma: (   12.7) %d\n", b);
	b=0; p =    "14.4"; a2i_sb  (&p,&b); printf ("     sb: (   14.4) %d\n", b);
	l=0; p =  "123456"; a2i_ra  (&p,&l); printf ("     ra: ( 123456)  %d ( 45296)\n", l);
	l=0; p = "-123456"; a2i_dec (&p,&l); printf ("    dec: (-123456) %d (-45296)\n", l);
	l=0; p = "-123456"; a2i_lat (&p,&l); printf ("    lat: (-123456) %d (-45296)\n", l);
	l=0; p = "1234567"; a2i_lon (&p,&l); printf ("    lon: (1234567) %d (445567)\n", l);
	b=0; p =     "179"; a2i_pa  (&p,&b); printf ("     pa: (    179) %d\n", b);

}
#endif
