/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <mpa.h>

/*
 *	Calculate Angular Separation of Two Coordinates
 *
 *		r1/d1 - RA/Dec in seconds of time/arc (integer)
 *		r2/d2 - RA/Dec in seconds of time/arc (integer)
 *
 *		return seconds of arc separation (integer)
 */
ang_sep (SL r1, SL d1, SL r2, SL d2)
{
	static	 fp64_t	 R1, D1;	/* [RA,Dec] base coordinate */
	static	 fp64_t	 S1, C1;	/* sin (D1), cos (D1) */
			 fp64_t	 R2, D2;	/* [RA,Dec] test coordinate */
			 fp64_t	 AD;		/* angular distance */
			ulong_t	 ad;

	if (r1 != ~0) { R1 = r1/3600.0*TWOPI/ 24.0; }
					R2 = r2/3600.0*TWOPI/ 24.0;
	if (d1 != ~0) { D1 = d1/3600.0*TWOPI/360.0; S1=sin (D1); C1=cos (D1); }
					D2 = d2/3600.0*TWOPI/360.0;

	AD = S1 * sin (D2) + C1 * cos (D2) * cos (R1 - R2);

	ad = 0.5 + acos (AD) * 3600.0/TWOPI*360.0;

	return (ad);
}

#ifdef TEST
main ()
{
	printf ("(10) %d\n",     ang_sep (0, 0, 0, 10));
	printf ("(118055) %d\n", ang_sep (770095, 69057, 724674, -40181));
}
#endif
