/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <mpa.h>
#include <gax_u2k.h>

#define	SecPerDay	86400

ulong_t	u2k_map (SL r, SL d)
{
	  ulong_t	 i = (d >= 0) ? 0 : 9 ;
	u2k_map_t	*p = &u2k_map_a [i];

	for ( ; i < U2K_MAP_N; i++, p++)
		if (d >= p->dec_n)
			break;

	if (p->cnt_n != 2) {
		r += (SecPerDay/2) / p->cnt_n;
		if (r >= SecPerDay)
		 	r -= SecPerDay;
	}

	return (r / (SecPerDay / p->cnt_n) + p->map_n);

}

ulong_t u2k_box (SL map, SL *rl, UL *ru, SL *dl, SL *du)
{
	   long_t	 i;
	u2k_map_t	*p = u2k_map_a;

	if (map > U2K_MAP_MAX) return (1);

	for (i = 0; i < U2K_MAP_N; i++, p++)
		if (map >= p->map_n && map < p->map_n + p->cnt_n)
			break;

	*dl  =      p->dec_n;
	*du  =  (p-1)->dec_n;
	*rl  = (p->cnt_n != 2) ? -(SecPerDay/2) / p->cnt_n : 0 ;
	*ru  = (p->cnt_n != 2) ? +(SecPerDay/2) / p->cnt_n : SecPerDay/2 ;
	*rl +=   SecPerDay / p->cnt_n * (map - p->map_n);
	*ru +=   SecPerDay / p->cnt_n * (map - p->map_n);

	if (*rl < 0) *rl += SecPerDay;

	return (0);
}

#ifdef	TEST

main ()
{
	UL	rl, ru;
	SL	dl, du;

	printf ("  1:%d\n", u2k_map (00 * 3600,        84 * 3600));
	printf ("  1:%d\n", u2k_map (12 * 3600 -    1, 84 * 3600));
	printf ("  2:%d\n", u2k_map (12 * 3600,        84 * 3600));
	printf ("  2:%d\n", u2k_map (24 * 3600 -    1, 84 * 3600));

	printf (" 35:%d\n", u2k_map (00 * 3600,        55 * 3600));
	printf (" 36:%d\n", u2k_map (00 * 3600 + 1800, 55 * 3600));
	printf (" 36:%d\n", u2k_map (01 * 3600,        55 * 3600));
	printf (" 37:%d\n", u2k_map (02 * 3600,        55 * 3600));
	printf (" 47:%d\n", u2k_map (12 * 3600,        55 * 3600));
	printf (" 58:%d\n", u2k_map (23 * 3600,        55 * 3600));
	printf (" 58:%d\n", u2k_map (23 * 3600 + 1799, 55 * 3600));
	printf (" 35:%d\n", u2k_map (23 * 3600 + 1800, 55 * 3600));
	printf (" 35:%d\n", u2k_map (24 * 3600 -    1, 55 * 3600));

	printf ("127:%d\n", u2k_map (01 * 3600 -    1, 17 * 3600));
	printf ("217:%d\n", u2k_map (01 * 3600 -    1, -5 * 3600));
	printf ("260:%d\n", u2k_map (00 * 3600       , -8 * 3600));

	u2k_box ( 01, &rl, &ru, &dl, &du);
	printf (" 01(1200,0000,84,90):(%02d%02d,%02d%02d,%02d,%02d)\n", 
		ru / 3600, ru / 60 % 60, rl / 3600, rl / 60 % 60,
		dl / 3600, du / 3600);

	u2k_box ( 15, &rl, &ru, &dl, &du);
	printf (" 15(0036,2324,61,72):(%02d%02d,%02d%02d,%02d,%02d)\n", 
		ru / 3600, ru / 60 % 60, rl / 3600, rl / 60 % 60,
		dl / 3600, du / 3600);

	u2k_box ( 35, &rl, &ru, &dl, &du);
	printf (" 35(0030,2330,50,61):(%02d%02d,%02d%02d,%02d,%02d)\n", 
		ru / 3600, ru / 60 % 60, rl / 3600, rl / 60 % 60,
		dl / 3600, du / 3600);

	u2k_box ( 73, &rl, &ru, &dl, &du);
	printf (" 73(1136,1048,39,50):(%02d%02d,%02d%02d,%02d,%02d)\n", 
		ru / 3600, ru / 60 % 60, rl / 3600, rl / 60 % 60,
		dl / 3600, du / 3600);

	u2k_box (223, &rl, &ru, &dl, &du);
	printf ("223(0432,0400,-6,06):(%02d%02d,%02d%02d,%02d,%02d)\n", 
		ru / 3600, ru / 60 % 60, rl / 3600, rl / 60 % 60,
		dl / 3600, du / 3600);
}

#endif
