/*
 *            Gax - Galaxy, Amateur Astronomical Database
 *       Copyright (C) 1998 - manders1@csc.com (Mark Anderson)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details in file GNU-GPL.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	GXD_h
#define	GXD_h

#include <gax_cat.h>

#define	NAN_B	0x80
#define	NAN_W	0x8000
#define	NAN_L	0x80000000

/*
 *	gxd_t - galaxy data binary structure
 */
typedef struct {
	 byte_t	 typ_n;		/* object type			 [TYP_tt ] */
	 byte_t	 cls_n;		/* object class w/i type [Ctt_ccc] */
	uword_t	 cat_x;		/* catalog map  w/i type [Xtt_aaa] */

	 long_t	 raj_n;		/* RA  J2000.0 (secs of time) */
	 long_t	 dcj_n;		/* DEC J2000.0 (secs of arc ) */

	uword_t	 maj_n;		/* major diameter arc '  [*10] */
	uword_t	 min_n;		/* minor diameter arc '  [*10] */
	ubyte_t	  mv_n;		/* magnitude (visual  )  [*10] */
	ubyte_t	  mp_n;		/* magnitude (photo   )  [*10] */
	ubyte_t	  ma_n;		/* magnitude (absolute)  [*10] */
	ubyte_t	  sb_n;		/* surface brightness    [*10] */
	ubyte_t	  pa_n;		/* position angle */

	ubyte_t	 con_n;		/* constellation */
#ifdef GX_Vel
	uword_t	 vel_n;		/* velocity */
#endif

	ulong_t	   m_a;		/* Messier Catalog */
	ulong_t	 ngc_a;		/* New General Catalog */
	ulong_t	  ic_a;		/* Index Catalog */
#ifdef GX_PGC
	ulong_t	 pgc_a;		/* Paturel Galaxy Catalog */
#endif
#ifdef GX_UGC
	ulong_t	 ugc_a;		/* Uppsala Galaxy Catalog */
#endif
#ifdef GX_MCG
	ulong_t	 mcg_a;		/* Morphological Catalogue of Galaxies */
#endif
#ifdef GX_ESO
	ulong_t	 eso_a;		/* European Southern Observatory Catalog */
#endif
#ifdef GX_ARP
	ulong_t	 arp_a;		/* Arp Catalog */
#endif
#ifdef GX_VV
	ulong_t	  vv_a;		/* VV Catalog */
#endif
#ifdef GX_MK
	ulong_t	  mk_a;		/* Markarian Catalog */
#endif
} gxd_t;

#endif
